<?php
if(!class_exists('OP_REST_API_Auth'))
{
    class OP_REST_API_Auth extends WC_REST_CRUD_Controller{
       
        protected $user_model;
        protected $register_model;
        protected $session_model;
        protected $namespace = 'op/v1';
        protected $session_class;
        protected $register_class;
        protected $warehouse_class;
        protected $op_core_class;
        public function __construct()
        {
            $this->user_model = new Op\Models\User();
            $this->register_model = new Op\Models\Register();
            $this->session_model = new Op\Models\Session();

            $this->session_class = new OP_Session();
            $this->register_class = new OP_Register();
            $this->warehouse_class = new OP_Warehouse();
            $this->op_core_class = new Openpos_Core();
        }
        public function register_routes() {
            
                // staff  route         
                register_rest_route( $this->namespace, '/auth/login', array(
                    'methods' => WP_REST_Server::CREATABLE,
                    'callback' => array($this,'login'),
                    'permission_callback' => '__return_true',
                    ) 
                );
                register_rest_route( $this->namespace, '/auth/login-register/(?P<id>\d+)', array(
                    'methods' => WP_REST_Server::CREATABLE,
                    'callback' => array($this,'login_register'),
                    'permission_callback' => '__return_true',
                    ) 
                );
                register_rest_route( $this->namespace, '/auth/login-session/(?P<id>\d+)', array(
                    'methods' => 'GET',
                    'callback' => array($this,'login_session'),
                    'permission_callback' => '__return_true',
                    ) 
                );
                register_rest_route( $this->namespace, '/auth/logout/(?P<id>\d+)', array(
                    'methods' => 'GET',
                    'callback' => array($this,'logout'),
                    'permission_callback' => array($this,'permission_callback'),
                    ) 
                );
                register_rest_route( $this->namespace, '/auth/logoff/(?P<id>\d+)', array(
                    'methods' => 'GET',
                    'callback' => array($this,'logoff'),
                    'permission_callback' => array($this,'permission_callback'),
                    ) 
                );
                register_rest_route( $this->namespace, '/auth/logon/(?P<id>\d+)', array(
                    'methods' => 'GET',
                    'callback' => array($this,'logon'),
                    'permission_callback' => array($this,'permission_callback'),
                    ) 
                );
        }
        public function login(WP_REST_Request $request){
            $result = array(
                'status' => 0,
                'data' => array(),
                'message' => ''
            );
            $response_code = 200;
            try{
                $username = $request->get_param( 'username' );
                $password = $request->get_param( 'password' );
                $login_mode = $request->get_param( 'login_mode' );
                $location = $request->get_param( 'location' );
                $lang = $request->get_param('lang');
                $time_stamp = $request->get_param('time_stamp');
                
                
                if($login_mode == 'pin')
                {
                    $user = $this->user_model->login_pin($password);
                }else{
                    $user = $this->user_model->login($username,$password);
                }

                if(is_wp_error($user))
                {
                   throw new Excepion($user->get_error_message());
                  
                }
                $registers = $this->register_class->registers();
                if(empty($registers))
                {
                    throw new Excepion("You not assign to any register");
                }
                $allow_registers = array();
                $user_id = $user->ID;
                foreach($registers as $register)
                {
                    $warehouse_id = $register['warehouse'];
                    
                    if($register['status'] == 'publish')
                    {
                        $warehouse = $this->warehouse_class->get($warehouse_id);
                        $state = '';
                        $country = '';
                        if($warehouse['country'])
                        {
                            if(strpos($warehouse['country'],':') !== false)
                            {
                                $tmp = explode(':',$warehouse['country']);
                                $state = end($tmp);
                                $country = $tmp[0];
                            }
                        }
                        $warehouse_address_data = array(
                            'address_1'  => $warehouse['address'],
                            'address_2'  => $warehouse['address_2'],
                            'city'       => $warehouse['city'],
                            'state'      => $state,
                            'postcode'   => $warehouse['postal_code'],
                            'country'    => $country,
                        );
                        
                        $cashiers = $register['cashiers'];
                        if(in_array($user_id,$cashiers))
                        {
                            if(!empty($warehouse))
                            {
                                $allow_registers[] = array(
                                    'id' => $register['id'],
                                    'name' => $register['name'],
                                    'mode' => $register['register_mode'],
                                    'outlet_id' => $warehouse['id'],
                                    'outlet_name' => $warehouse['name'],
                                    'address' => WC()->countries->get_formatted_address( $warehouse_address_data,',' )
                                );
                            }
                            
                        }
                    }
                }

                $ip = $this->op_core_class->getClientIp();

                $avatar = rtrim(OPENPOS_URL,'/').'/assets/images/default_avatar.png';

                $avatar_args = get_avatar_data( $user_id);
                if($avatar_args && isset($avatar_args['url']))
                {
                    $avatar = $avatar_args['url'];
                }
                $prefix = 'api-'.sanitize_title($user->user_login);
                $session_id = $this->session_class->generate_session_id($prefix);
                
                $result['data']['session'] = $session_id;
                $result['data']['user'] = array(
                    'user_id' => $user_id,
                    'username' => $user->data->user_login,
                    'avatar' => $avatar,
                    'name' => $user->data->display_name,
                    'email' => $user->data->user_email,
                    'phone' => '',
                    'role' => $user->roles,
                    'login_location' => $location,
                    'login_ip' => $ip,
                    'login_time' => current_time('Y-m-d H:i:s',true),
                );
                $result['data']['cash_drawers'] = $allow_registers;
                $this->session_class->save($session_id,$result['data'] );
                $result['status'] = 1;
                
            }catch(Exception $e){
                $result['message'] = $e->getMessage();
                $result['status'] = 0;
                $response_code = 400;
                return new WP_Error( 'login_error', $e->getMessage() , array('status' => 404) );
            }
            $response = rest_ensure_response( $result );
            $response->set_status( $response_code );
            return $response;
        }
        public function login_register(WP_REST_Request $request){
            
            $response_code = 200;
            $result = array(
                'status' => 0,
                'data' => array(),
                'message' => ''
            );
            try{
                $session = $request->get_param( 'session' );
                $client_time_offset = $request->get_param( 'client_time_offset' );

            }catch(Exception $e){
                $result['message'] = $e->getMessage();
                $result['status'] = 0;
                $response_code = 400;
                return new WP_Error( 'login_error', $e->getMessage() , array('status' => 404) );
            }

            $response = rest_ensure_response( $result );
            $response->set_status( $response_code );
            return $response;

        }
        public function logon(WP_REST_Request $request){
            $response_code = 200;
            $result = array(
                'status' => 0,
                'data' => array(),
                'message' => ''
            );
            try{


            }catch(Exception $e){
                $result['message'] = $e->getMessage();
                $result['status'] = 0;
                $response_code = 400;
                return new WP_Error( 'login_error', $e->getMessage() , array('status' => 404) );
            }

            $response = rest_ensure_response( $result );
            $response->set_status( $response_code );
            return $response;
        }
        public function logoff(WP_REST_Request $request){
            $response_code = 200;
            $result = array(
                'status' => 0,
                'data' => array(),
                'message' => ''
            );
            try{


            }catch(Exception $e){
                $result['message'] = $e->getMessage();
                $result['status'] = 0;
                $response_code = 400;
                return new WP_Error( 'login_error', $e->getMessage() , array('status' => 404) );
            }

            $response = rest_ensure_response( $result );
            $response->set_status( $response_code );
            return $response;
        }
        public function login_session(WP_REST_Request $request){
            $response_code = 200;
            $result = array(
                'status' => 0,
                'data' => array(),
                'message' => ''
            );
            try{


            }catch(Exception $e){
                $result['message'] = $e->getMessage();
                $result['status'] = 0;
                $response_code = 400;
                return new WP_Error( 'login_error', $e->getMessage() , array('status' => 404) );
            }

            $response = rest_ensure_response( $result );
            $response->set_status( $response_code );
            return $response;
        }
        public function logout(){
            $response_code = 200;
            $result = array(
                'status' => 0,
                'data' => array(),
                'message' => ''
            );
            try{


            }catch(Exception $e){
                $result['message'] = $e->getMessage();
                $result['status'] = 0;
                $response_code = 400;
                return new WP_Error( 'login_error', $e->getMessage() , array('status' => 404) );
            }

            $response = rest_ensure_response( $result );
            $response->set_status( $response_code );
            return $response;
        }
        
    }
}